% helper function to solve for a and e of W120-ellipse

% input: var (1 x 2 double vector) containing: 
%             a (double) semi-major axis of equivalent ellipse of W120
%             e (double) sqrt(1-semi-minor^2/semi-major^2) equiv. ell. of W120
%        eigenvals (2 x 2 double matrix of W120 eigenvalues)
%
% output: Eq.(13) of  Rahman & Shandarin, MNRAS 343 (2003)
%                  with P_ell brought to right hand side

function [eq_13] = ellipse_equations_W120(var, eigenvals)

% define names for entries of variable vector
% (use e^2 as variable instead of e, as all functions depend on e^2 only)
a = var(1,1);
esquared = var(1,2); 

% elliptical integrals
syms varphi;
E_e = int(sqrt(1-esquared*(sin(varphi))^2),varphi,0,pi/2);
K_e = int(1/sqrt(1-esquared*(sin(varphi))^2),varphi,0,pi/2);

% Eq. (13) of Rahman & Shandarin, MNRAS 343 (2003) with P_ell brought to rhs
eq_13(1) = double(4/3*a^3/(esquared)*((1-esquared^2)*E_e-(1-esquared)^2*K_e) - eigenvals(1,1));
eq_13(2) = double(4/3*a^3/(esquared)*((1-esquared)*K_e-(1-2*esquared)*E_e) - eigenvals(2,2));


% attempt to retrieve symbolic solution (did not work out)
% syms a;
% syms esquare;
% syms sinvarphi;
% assume(a,'real');
% assumeAlso(a,'positive');
% assume(esquare,'real');
% assumeAlso(esquare < 1);
% assume(sinvarphi,'positive');
% % rewrite integral according to GR p860 2. and 3.
% E_e = int(sqrt(1-esquare*(sinvarphi)^2)/sqrt(1-sinvarphi^2),sinvarphi,0,1);
% K_e = int(1/sqrt((1-sinvarphi^2)*(1-esquare*(sinvarphi)^2)),sinvarphi,0,1);
% % E_e = int(sqrt(1-(e*sin(varphi))^2),varphi,0,pi/2);
% % K_e = int(1/sqrt(1-(e*sin(varphi))^2),varphi,0,pi/2);
% sol = solve([eigenvals(1,1) == 4/3*a^3/(esquare)*((1-esquare^2)*E_e-(1-esquare)^2*K_e), eigenvals(2,2) == 4/3*a^3/(esquare)*((1-esquare)*K_e-(1-2*esquare)*E_e)], [a, esquare]);